import chattertools as ch
import os
import time
from datetime import datetime
from dotenv import load_dotenv
load_dotenv()

class MachineWatcher:
    def __init__(self, machineId):
        self.machineId = machineId
        self.chatter = ch.Client(key = os.getenv('CHATTER_API_KEY'))

        self.state = {
            ch.DataType.ONLINE: None,
            ch.DataType.MODE: None,
            ch.DataType.PROGRAM: None,
            ch.DataType.SPINDLE_SPEED: None,
            ch.DataType.PART_COUNT: None,
            ch.DataType.PART_GOAL: None,
        }

    def run(self):
        while True:
            externalState = self.chatter.machines.get(1025).state.get()
            for transaction in externalState:
                if transaction.dataType in self.state:
                    self.state[transaction.dataType] = transaction.value

            print(f'Machine {self.machineId} state - {datetime.now()}')
            for key, value in self.state.items():
                print(f'{key}: {value}')

            time.sleep(1)

if __name__ == "__main__":
    watcher = MachineWatcher(1025)
    try:
        watcher.run()
    except KeyboardInterrupt:
        print('Exiting...')
        exit()


import chattertools as ch

# Create the focas object and connect to the machine
focas = ch.Focas(
    ip='10.0.2.51', # IP of the machine     * Required
    port=8193,      # Port number           Default: 8193
    timeout=3       # Timeout in seconds    Default: 3
)

# Get the program name
response = focas.cnc_exeprgname()
print('Program Name:', response.name)
print('Program Number:', response.oNumber)
print('Program Path', focas.cnc_exeprgname2())

# Disconnect from the machine
focas = None



import ctypes

class ODBM(ctypes.Structure):
    _fields_ =[
        ('datano', ctypes.c_short),
        ('dummy', ctypes.c_short),
        ('mcr_val', ctypes.c_long),
        ('dec_val', ctypes.c_long)
    ]


class ODBEXEPRG(ctypes.Structure):
    _pack_ = 4
    _fields_ = [("name", ctypes.c_char * 36),
                ("oNumber", ctypes.c_long), ]
    
    def getPyObj(self):
        return odbexeprg(self.name, self.oNumber)
    
class odbexeprg():
    def __init__(self, name, oNumber):
        self.name = name.decode('ascii').strip()
        self.oNumber = int(oNumber)

    def __str__(self):
        return f'obdexeprg(name={self.name}, oNumber={self.oNumber})'


class ODBST(ctypes.Structure):
    _fields_ = [
        ('dummy', ctypes.c_short * 2),
        ('aut', ctypes.c_short),
        ('manual', ctypes.c_short),
        ('run', ctypes.c_short),
        ('edit', ctypes.c_short),
        ('motion', ctypes.c_short),
        ('mstb', ctypes.c_short),
        ('emergency', ctypes.c_short),
        ('write', ctypes.c_short),
        ('labelskip', ctypes.c_short),
        ('alarm', ctypes.c_short),
        ('warning', ctypes.c_short),
        ('battery', ctypes.c_short),
    ]

    def getPyObj(self):
        return odbst(self.aut, self.manual, self.run, self.edit, self.motion, self.mstb, self.emergency, self.alarm, self.write, self.labelskip, self.warning, self.battery)
    
class odbst():
    def __init__(self, aut, manual, run, edit, motion, mstb, emergency, alarm, write, labelskip, warning, battery):
        self.aut = self._getMapping('aut', aut)
        self.manual = self._getMapping('manual', manual)
        self.run = self._getMapping('run', run)
        self.edit = self._getMapping('edit', edit)
        self.motion = self._getMapping('motion', motion)
        self.mstb = self._getMapping('mstb', mstb)
        self.emergency = self._getMapping('emergency', emergency)
        self.alarm = self._getMapping('alarm', alarm)
        self.write = self._getMapping('write', write)
        self.labelskip = self._getMapping('labelskip', labelskip)
        self.warning = self._getMapping('warning', warning)
        self.battery = self._getMapping('battery', battery)

    def _getMapping(self, key, value):
        mappings = {
            'aut': [None, 'MDI', 'DNC', 'MEM', 'EDIT', 'TH IN'],
            'manual': [None, 'REF', 'INC', 'HND', 'JOG', 'AGJ', 'I+H', 'J+H'],
            'run': ['STOP', 'HOLD', 'STRT', 'MSTR', 'RSTR', 'PRSR', 'NSRC', 'RSTR', 'RSET', None, None, None, None, 'HPCC'],
            'edit': [None, 'EDIT', 'SRCH', 'VRFY', 'COND', 'READ', 'PNCH'],
            'motion': [None, 'MTN', 'DWL', 'W'],
            'mstb': [None, 'FIN'],
            'emergency': [False, True],
            'write': [False, True],
            'labelskip': [True, False],
            'alarm': [False, True],
            'warning': [False, True],
            'battery': [False, True, True]
        }

        if (key in mappings):
            if (value < len(mappings[key])):
                return mappings[key][value]
            else:
                raise ValueError(f'No mapping found for key {key} at value {value}')
        else:
            raise ValueError(f'No mapping found for {key}')

    def __str__(self):
        return f'odbst(aut={self.aut}, manual={self.manual}, run={self.run}, edit={self.edit}, motion={self.motion}, mstb={self.mstb}, emergency={self.emergency}, alarm={self.alarm}, write={self.write}, labelskip={self.labelskip}, warning={self.warning}, battery={self.battery})'

class ODBACT(ctypes.Structure):
    _fields_ = [
        ('dummy', ctypes.c_short * 2),
        ('data', ctypes.c_long)
    ]

    def getPyObj(self):
        return odbact(self.data)

class odbact():
    def __init__(self, data):
        self.data = int(data)

    def __str__(self):
        return f'odbact(data={self.data})'

class OPMSG(ctypes.Structure):
    _fields_ = [
        ('datano', ctypes.c_short),
        ('type', ctypes.c_short),
        ('char_num', ctypes.c_short),
        ('data', ctypes.c_char * 256)
    ]

class ODBALMMSG(ctypes.Structure):
    _fields_ = [
        ("alm_no", ctypes.c_long),
        ("type", ctypes.c_short),
        ("axis", ctypes.c_short),
        ("dummy", ctypes.c_short),
        ("msg_len", ctypes.c_short),
        ("alm_msg", ctypes.c_char * 32)
    ]

    def getPyObj(self):
        return odbalmmsg(self.alm_no, self.type, self.axis, self.msg_len, self.alm_msg)

class odbalmmsg():
    def __init__(self, alm_no, type, axis, msg_len, alm_msg):
        self.alm_no = int(alm_no)
        self.type = int(type)
        self.axis = int(axis)
        self.msg_len = int(msg_len)
        self.alm_msg = alm_msg.decode('ascii').strip()

    def __str__(self):
        return f'odbalmmsg(alm_no={self.alm_no}, type={self.type}, axis={self.axis}, msg_len={self.msg_len}, alm_msg={self.alm_msg})'


import unittest
import os
from dotenv import load_dotenv
import chattertools as ch
from chattertools.models import Transaction


class TestTransactions(unittest.TestCase):

    def test_create_valid_transactions(self):
        transactions = [
            Transaction(None, ch.DataType.MODE, 'MEM'),
            Transaction(None, ch.DataType.PART_COUNT, 100),
            Transaction(None, ch.DataType.ALARM_ACTIVE, True),
            Transaction(None, ch.DataType.ALARM_CONTENTS, 'Alarm Contents'),
            Transaction(None, ch.DataType.INSPECTION_SIZE, 1.23),
        ]
        self.assertTrue(len(transactions) > 0)


from typing import List

from .macro import Macro
from ..services.macro_service import MacroService
from .transaction import Transaction
from ..services.transaction_service import TransactionService
from ..services.state_service import StateService
from .transaction import Transaction

class Machine:
    def __init__(self, requestor, id: int, name: str):
        self._requestor = requestor
        self.id = id
        self.name = name

        self._state = None
        self._transactions = None
        self._macros = None

    @property
    def state(self) -> List[Transaction]:
        if self._state is None:
            self._state = StateService(self._requestor, self.id)
        return self._state
    
    @property
    def transactions(self) -> List[Transaction]:
        if self._transactions is None:
            self._transactions = TransactionService(self._requestor, self.id)
        return self._transactions

    @property
    def macros(self) -> List[Macro]:
        if self._macros is None:
            self._macros = MacroService(self._requestor, self.id)
        return self._macros

    @classmethod
    def fromDict(cls, requestor, data: dict) -> 'Machine':
        return cls(requestor, data['id'], data['name'])
    
    def toDict(self) -> dict:
        return {
            "id": self.id,
            "name": self.name
        }
    
    def __str__(self):
        return f"Machine {self.id}: {self.name}"


import chattertools as ch

# Create the focas object and connect to the machine
focas = ch.Focas(
    ip='10.0.2.51', # IP of the machine     * Required
    port=8193,      # Port number           Default: 8193
    timeout=3       # Timeout in seconds    Default: 3
)

MACRO_VAR = 101
NEW_VAL = 123.456

# Read the value of the macro variable
print(f'Macro {MACRO_VAR} value:', focas.cnc_rdmacro(MACRO_VAR))

# Set the value of the macro variable
focas.cnc_wrmacro(MACRO_VAR, NEW_VAL)

# Read the new value of the macro variable
print(f'Macro {MACRO_VAR} new value:', focas.cnc_rdmacro(MACRO_VAR))

# Disconnect from the machine
focas = None


class Macro:
    def __init__(self, id: int, macroVariable: int):
        self.id = id
        self.macroVariable = macroVariable

    @staticmethod
    def fromDict(data: dict) -> 'Macro':
        return Macro(data['id'], data['macroVariable'])
    
    def toDict(self) -> dict:
        return {
            "id": self.id,
            "macroVariable": self.macroVariable
        }
    
    def __str__(self):
        return f"Macro {self.id}: {self.macroVariable}"


from datetime import datetime
from typing import Any, Union

def parseInt(val: Any) -> int:
    if val is None: return None
    try:
        return int(val)
    except ValueError:
        raise ValueError(f"Cannot convert value {val} to integer")

def parseDatetime(val: Union[datetime, int, str]) -> datetime:
    if val is None: return None

    if isinstance(val, datetime): return val
    if isinstance(val, int): return datetime.fromtimestamp(val)

    if isinstance(val, str):
        formats = ["%Y-%m-%dT%H:%M:%S.%fZ", "%Y-%m-%d %H:%M:%S"]
        for format in formats:
            try:
                return datetime.strptime(val, format)
            except ValueError:
                pass

        raise ValueError(f'Invalid datetime format "{val}"')
    
def parseApiDatetime(val: datetime) -> str:
    return val.strftime("%Y-%m-%d %H:%M:%S")
    
def parseBool(val: Union[bool, int, str]) -> bool:
    if val is None: return None
    if isinstance(val, bool): return val
    if isinstance(val, int):
        try:
            return bool(val)
        except ValueError:
            pass
    if isinstance(val, str): return val.lower() == "true"
    raise ValueError(f"Cannot convert value {val} to boolean")


from datetime import datetime
from ..helpers.parsers import parseDatetime, parseInt, parseBool, parseApiDatetime
from .data_type import DataType
from typing import Any

class Transaction:
    def __init__(self, timestamp: datetime, dataType: DataType, value: Any):
        self.timestamp = parseDatetime(timestamp) if timestamp else datetime.now()
        self.dataType = dataType
        self.value = self.enforceType(dataType, value)

    @staticmethod
    def fromDict(data: dict) -> 'Transaction':
        return Transaction(
            data['timestamp'],
            DataType.fromValue(int(data['dataType'])),
            str(data['value'])
        )
    
    def toDict(self) -> dict:
        return {
            "timestamp": parseApiDatetime(self.timestamp),
            "dataType": self.dataType.value,
            "value": str(self.value)
        }
    
    @staticmethod
    def enforceType(dataType: DataType, value):
        if value is None:
            return None
        parser = DataType.getParser(dataType.type)
        value = parser(value) if parser else value
        if not isinstance(value, dataType.type):
            raise TypeError(f"Value {value} is not of type {dataType.type}, got {type(value)} instead")
        return value
    
    def __str__(self):
        return f"Transaction {self.timestamp}: {self.dataType} - {self.value}"


from .chatter_requestor import ChatterRequestor
from .services.machine_service import MachineService

class Client:
    def __init__(self, key: str, baseUrl: str = "https://apiv2.chatter.dev"):
        if key is None:
            raise ValueError("An API key is required")
        self.key = key
        self.baseUrl = baseUrl
        self._requestor = ChatterRequestor(key, baseUrl)

        # Services
        self.machines = MachineService(self._requestor)


from typing import List
from ..models.macro import Macro
from .service import Service

class MacroService(Service):
    def __init__(self, requestor, machineId: int, *args, **kwargs):
        super().__init__(requestor, *args, **kwargs)
        
        self.machineId = machineId

    def list(self, limit: int = 100, offset: int = 0) -> List[Macro]:
        """
        List all macros
        """
        params = {
            "limit": limit,
            "offset": offset
        }
        data = self._requestor.get(f"/machines/{self.machineId}/macros", params=params)
        return [Macro.fromDict(macro) for macro in data]
    
    def get(self, id: int) -> Macro:
        raise NotImplementedError("Method not implemented")
        """
        Get a macro by ID
        """
        data = self._requestor.get(f"/machines/{id}/macros/{id}")
        return Macro.fromDict(data)
    
    def create(self, macroVariable: int) -> Macro:
        raise NotImplementedError("Method not implemented")
        """
        Create a new macro
        """
        data = macroVariable.toDict()
        data = self._requestor.post(f"/machines/{self.machineId}/macros", data=data)
        return Macro.fromDict(data)
    
    def delete(self, id: int) -> None:
        """
        Delete a macro by ID
        """
        self._requestor.delete(f"/machines/{self.machineId}/macros/{id}")



import unittest
import chattertools as ch
import os

class TestFocas(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.focas = ch.Focas(ip='10.0.2.51', port=8193, timeout=3)

    def test_cnc_allclibhndl3(self):
        self.assertTrue(self.focas.handle != 0)

    def test_cnc_exeprgname(self):
        response = self.focas.cnc_exeprgname()
        self.assertTrue(response != None)

    def test_acts(self):
        response = self.focas.cnc_acts()
        self.assertTrue(response != None)

    def test_cnc_statinfo(self):
        response = self.focas.cnc_statinfo()
        self.assertTrue(response != None)

    def test_cnc_rdalmmsg(self):
        response = self.focas.cnc_rdalmmsg()
        self.assertTrue(response != None)

    def test_rdopmsg3(self):
        response = self.focas.cnc_rdopmsg3()
        self.assertTrue(response != None)

    def test_cnc_rdmacro(self):
        MACRO_VAR = 101
        response = self.focas.cnc_rdmacro(MACRO_VAR)

    def test_cnc_wrmacro(self):
        MACRO_VAR = 102
        TEST_VALUE = 1234.5678
        self.focas.cnc_wrmacro(MACRO_VAR, TEST_VALUE)
        macroRead = self.focas.cnc_rdmacro(MACRO_VAR)
        self.assertTrue(macroRead == TEST_VALUE)

    @classmethod
    def tearDownClass(self):
        self.focas = None
        if os.path.exists('focas.log'):
            os.remove('focas.log')

if __name__ == '__main__':
    unittest.main()


import unittest
from datetime import datetime
from chattertools.helpers.parsers import parseDatetime

class TestParseDatetime(unittest.TestCase):

    def test_with_datetime(self):
        """Test with a datetime object."""
        input_val = datetime(2022, 1, 1, 0, 0)
        expected = input_val
        self.assertEqual(parseDatetime(input_val), expected)

    def test_with_unix_timestamp(self):
        """Test with a Unix timestamp."""
        input_val = 1640995200  # Equivalent to datetime(2022, 1, 1, 0, 0)
        expected = datetime.fromtimestamp(input_val)
        self.assertEqual(parseDatetime(input_val), expected)

    def test_with_correct_string(self):
        """Test with a correctly formatted string."""
        input_val = "2022-01-01T00:00:00.000Z"
        expected = datetime(2022, 1, 1, 0, 0)
        self.assertEqual(parseDatetime(input_val), expected)

    def test_with_incorrect_string(self):
        """Test with an incorrectly formatted string."""
        input_val = "not-a-valid-date"
        with self.assertRaises(ValueError):
            parseDatetime(input_val)

if __name__ == '__main__':
    unittest.main()



import os
import platform
from pathlib import Path
from ctypes import *

from chattertools.focas_structs import *

class Focas:
    def __init__(self, ip: str, port: int = 8193, timeout: int = 3, logfile: str = 'focas.log'):
        self.config = {
            'ip': ip,
            'port': port,
            'timeout': timeout,
            'logfile': logfile
        }
        
        self.handle = c_ushort(0)

        self._loadDrivers()

        if self._isPlatformLinux():
            # Linux requires a logfile to exist while the connection is open, and a setup process to be run
            logfileName = self.config.get('logfile')
            self.fwlib.cnc_startupprocess(c_short(1), c_char_p(logfileName.encode('ascii')))

        self._getHandle()

    def __del__(self):
        if self.handle.value:
            self._releaseHandle()
        if self._isPlatformLinux():
            # Linux requires the exitprocess to be run when the connection is closed
            self.fwlib.cnc_exitprocess()

    def _getHandle(self):
        self.handle = self.cnc_allclibhndl3(self.config.get('ip'), self.config.get('port'), self.config.get('timeout'))

    def _releaseHandle(self):
        self.cnc_freelibhndl()

    # FOCAS Methods
    def cnc_allclibhndl3(self, ip: str, port: int, timeout: int = 10):
        """ Allocates the library handle and connects to CNC that has the specified IP address or the Host Name.
        Args:
            ip (str): The IP address of the CNC to connect to
            port (int): The port number to connect to
            timeout (int): The timeout for the connection
        Omitted Fanuc Args:
            handle: Pointer to the handle to be allocated
        Returns:
            int: The result of the connection
        """
        func = self.fwlib.cnc_allclibhndl3
        func.restype = c_short
        handle = c_ushort(0)
        result = func(ip.encode('ascii'), port, timeout, byref(handle))
        self._validateResponse(result)
        return handle
    
    def cnc_freelibhndl(self):
        """ Frees the library handle which was used by the Data window library.
        Args:
            None
        Omitted Fanuc Args:
            handle: Handle to be freed
        Returns:
            None
        """
        func = self.fwlib.cnc_freelibhndl
        func.restype = c_short
        result = func(self.handle)
        self._validateResponse(result)

    def cnc_exeprgname(self):
        """ Reads full path name of the program which is being currently executed in CNC.
            When the CNC is stopping, the name of the executed program is acquired.
            The program name is stored in "execprg.name" with maximum 32 character string format.
        Args:
            None
        Omitted Fanuc Args:
            handle: Handle to be freed
            exeprg: Structure to store the program name
        Returns:
            odbexeprg (class):
                name (str): The name of the program
                oNumber (int): The program number
        """
        f = self.fwlib.cnc_exeprgname
        f.argtypes = [c_ushort, POINTER(ODBEXEPRG)]
        f.restype = c_short

        exeprg = ODBEXEPRG()

        result = f(self.handle, byref(exeprg))
        self._validateResponse(result)

        return exeprg.getPyObj()
    
    def cnc_exeprgname2(self):
        """ Reads full path name of the program which is being currently executed in CNC.
            When the CNC is stopping, the name of the executed program is acquired.
            The program name is stored in "path_name" with maximum 256 character string format.
        Args: None
        Omitted Fanuc Args:
            handle
            char *path_name
        Returns:
            str: The full path of the program
        """
        f = self.fwlib.cnc_exeprgname2
        f.argtypes = [c_ushort, POINTER(c_char * 256)]
        f.restype = c_short

        path = (c_char * 256)()

        result = f(self.handle, byref(path))
        self._validateResponse(result)

        return path.value.decode('ascii').strip()
    
    def cnc_acts(self):
        """ Reads the actual rotational speed of the spindle connected to CNC.
            The actual spindle speed is stored in "data" of "ODBACT".
        Args:
            None
        Omitted Fanuc Args:
            handle: The handle
            odbact: Structure to store the spindle speed
        Returns:
            odbact (class):
                data (int): The actual spindle speed
        """
        f = self.fwlib.cnc_acts
        f.argtypes = [c_ushort, POINTER(ODBACT)]
        f.restype = c_short

        speed = ODBACT()

        result = f(self.handle, byref(speed))
        self._validateResponse(result)

        return speed.getPyObj()
    
    def cnc_statinfo(self):
        """ Reads the status information of CNC. The various information is stored in each member of "ODBST".
        
        Args:
            None
        Omitted Fanuc Args:
            handle: The handle
            ODBST: Structure to store the status information
        Returns:
            ODBST (class):
                aut (bool): AUTOMATIC mode selection
                manual (bool): MANUAL mode selection
                run (bool): Status of automatic operation
                edit (bool): Status of program editing
                motion (bool): Status of axis movement,dwell
                mstb (bool): Status of M,S,T,B function
                emergency (bool): Status of emergency
                write (bool): Status of writing backupped memory
                labelskip (bool): Status of label skip
                alarm (bool): Status of alarm
                warning (bool): Status of warning
                battery (bool): Status of battery
        """

        f = self.fwlib.cnc_statinfo
        f.argtypes = [c_ushort, POINTER(ODBST)]
        f.restype = c_short

        status = ODBST()

        result = f(self.handle, byref(status))
        self._validateResponse(result)

        return status.getPyObj()
    
    def cnc_rdalmmsg(self):
        f = self.fwlib.cnc_rdalmmsg
        f.argtypes = [c_ushort, c_short, POINTER(c_short), POINTER(ODBALMMSG)]
        f.restype = c_short

        quantity = c_short(10)
        buffer = ODBALMMSG()

        result = f(self.handle, c_short(-1), byref(quantity), byref(buffer))
        self._validateResponse(result)

        return buffer.getPyObj()
    
    def cnc_rdopmsg3(self):
        f = self.fwlib.cnc_rdopmsg3
        f.argtypes = [c_ushort, c_short, POINTER(c_short), POINTER(OPMSG)]
        f.restype = c_short

        buffer = OPMSG()

        sequenceNumber = c_short(0)
        numberRead = c_short(1)

        ret = []

        while True:
            result = f(self.handle, sequenceNumber, byref(numberRead), byref(buffer))
            self._validateResponse(result)
            msg = buffer.data.decode('ascii').strip()
            if len(msg) == 0 or msg is None:
                break
            ret.append(msg)
            sequenceNumber.value += 1

        return ret
    
    def cnc_rdmacro(self, number: int):
        f = self.fwlib.cnc_rdmacro
        f.argtypes = [c_ushort, c_short, c_short, POINTER(ODBM)]
        f.restype = c_short

        macro = ODBM()
        
        result = f(self.handle, c_short(number), c_short(12), byref(macro))
        self._validateResponse(result)

        return self.decodeFanucMacro(macro)
    
    def cnc_wrmacro(self, number: int, value: float):
        f = self.fwlib.cnc_wrmacro
        f.argtypes = [c_ushort, c_short, c_short, c_long, c_short]
        f.restype = c_short

        result = f(self.handle, c_short(number), c_short(10), *self.encodeFanucMacro(value))
        self._validateResponse(result)

        return True

    @staticmethod
    def decodeFanucMacro(input: ODBM):
        macroVal = input.mcr_val
        decVal = input.dec_val

        if decVal in [-1, 65535]:
            return None # Vacant / Null variable in the control

        absValStr = f"{abs(macroVal):09d}"

        if 0 <= decVal <= 9:
            # Insert decimal point at the correct location
            valStr = absValStr[:9-decVal] + '.' + absValStr[9-decVal:]
        elif decVal > 9:
            # Insert decimal point at the start and then scale
            valStr = "." + absValStr
            value = float(valStr) / (10 ** (decVal - 9))
            if macroVal < 0:
                value = -value
            return value

        if macroVal < 0:
            valStr = "-" + valStr

        return float(valStr)
    
    @staticmethod
    def encodeFanucMacro(value: float):
        if value is None:
            # Handle null value
            return c_long(0), c_short(-1)

        # Convert the float to a string
        value_str = str(value)

        # Remove the decimal point and count how many places it moved
        if '.' in value_str:
            dot_index = value_str.index('.')
            value_str = value_str.replace('.', '')
            dec_val = dot_index
        else:
            dec_val = len(value_str)

        # Adjust the string length to 9 digits
        value_str = value_str[:9].ljust(9, '0')
        
        # Calculate mcr_val
        mcr_val = int(value_str)

        # Adjust dec_val to reflect the original decimal position
        dec_val = len(value_str) - dec_val

        return c_long(mcr_val), c_short(dec_val)

    def _loadDrivers(self):
        if self._isPlatformWindows():
            BASE_DIR = Path(__file__).resolve().parent
            DRIVER_DIR = os.path.join(BASE_DIR, 'lib', 'Fwlib64')
            DRIVER_NAME = 'Fwlib64'
            DRIVER_PATH = os.path.join(BASE_DIR, DRIVER_DIR, DRIVER_NAME + '.dll')

            if not os.path.exists(DRIVER_PATH):
                raise Exception(f"Driver not found at {DRIVER_PATH}")
            self.fwlib = windll.LoadLibrary(DRIVER_PATH)

            for file in os.listdir(DRIVER_DIR):
                if file.endswith('.dll'):
                    windll.LoadLibrary(os.path.join(DRIVER_DIR, file))

        elif self._isPlatformLinux():
            BASE_DIR = Path(__file__).resolve().parent
            DRIVER_DIR = './lib/'
            DRIVER_BASENAME = 'libfwlib32-linux'
            DRIVER_VERSION = '1.0.5'

            machine = platform.machine()
            if machine == 'armv7l':
                DRIVER_SUFFIX = '-armv7'
            elif machine == 'x86_64':
                DRIVER_SUFFIX = '-x64'
            elif machine == 'i386':
                DRIVER_SUFFIX = '-x86'
            else:
                raise Exception('Unsupported architecture: ' + machine)

            DRIVER_PATH = os.path.join(BASE_DIR, DRIVER_DIR, DRIVER_BASENAME + DRIVER_SUFFIX + '.so.' + DRIVER_VERSION)
            
            if not os.path.exists(DRIVER_PATH):
                raise Exception(f"Driver not found at {DRIVER_PATH}")
            self.fwlib = cdll.LoadLibrary(DRIVER_PATH)

        else:
            raise Exception('Unsupported platform: ' + platform.system())

    def _validateResponse(self, value):
        if value == 0 and value is not None:
            return
        errors = {
            -17: "Protocol Error (EW_PROTOCOL)",
            -16: "Socket error (EW_SOCKET)",
            -15: "DLL Not Found (EW_NODLL)",
            -11: "Bus error (EW_BUS)",
            -10: "System error (EW_SYSTEM2)",
            -9: "HSSB Serial error (EW_HSSB)",
            -8: "Handle error (EW_HANDLE)",
            -7: "CNC/PMC Version Mismatch (EW_VERSION)",
            -6: "Abnormal error??? (EW_UNEXP)",
            -5: "System error (EW_SYSTEM)",
            -4: "Shared RAM parity error (EW_PARITY)",
            -3: "EMM386 or mmcsys install error (EW_MMCSYS)",
            -2: "Reset or stop occurred (EW_RESET)",
            -1: "Busy error (EW_BUSY)",
            0: "No error (EW_OK)",
            1: "'Command prepare error (EW_FUNC)' or 'PMC does not exist (EW_NOPMC)'",
            2: "Data block length error (EW_LENGTH)",
            3: "Data number error (EW_NUMBER)",
            4: "'Data attribute error (EW_ATTRIB)' or 'Data type error (EW_TYPE)'",
            5: "Data error (EW_DATA)",
            6: "'No option' error (EW_NOOPT)",
            7: "Write protect error (EW_PROT)",
            8: "Memory overflow error (EW_OVRFLOW)",
            9: "CNC Parameter not correct (EW_PARAM)",
            10: "Buffer error (EW_BUFFER)",
            11: "Path error (EW_PATH)",
            12: "CNC Mode error (EW_MODE)",
            13: "Execution rejected (EW_REJECT)",
            14: "Data server error (EW_DTSRVR)",
            15: "Alarm occurred (EW_ALARM)",
            16: "CNC is not running (EW_STOP)",
            17: "Protection data error (EW_PASSWD)",
            18: "Error generated by PMC (EW_PMC)",
            19: "PMC Handle error (EW_PMCHANDLE)"
        }

        if value in errors:
            raise Exception(errors[value], value)
        else:
            message = f"An unknown value was provided to FocasException: {value}"
            raise Exception(message, value)

    @staticmethod
    def _isPlatformLinux():
        return platform.system() == 'Linux'
    
    @staticmethod
    def _isPlatformWindows():
        return platform.system() == 'Windows'


from typing import List
from ..models.transaction import Transaction
from .service import Service

from datetime import datetime

class TransactionService(Service):
    def __init__(self, requestor, machineId: int, *args, **kwargs):
        super().__init__(requestor, *args, **kwargs)
        
        self.machineId = machineId

    def list(self, limit: int = 100, offset: int = 0) -> List[Transaction]:
        """
        List all transactions
        """
        params = {
            "limit": limit,
            "offset": offset
        }
        data = self._requestor.get(f"/machines/{self.machineId}/transactions", params=params)
        return [Transaction.fromDict(t) for t in data]
    
    def get(self, id: int) -> Transaction:
        raise NotImplementedError("Method not implemented")
        """
        Get a transaction by ID
        """
        data = self._requestor.get(f"/machines/{id}/transactions/{id}")
        return Transaction.fromDict(data)
    
    def create(self, timestamp: datetime, dataType: int, value: str) -> Transaction:
        """
        Create a new transaction
        """
        transaction = Transaction(timestamp, dataType, value)
        data = self._requestor.post(f"/machines/{self.machineId}/transactions", data=transaction.toDict())
        return Transaction.fromDict(data)


import unittest
import os
from dotenv import load_dotenv
import chattertools as ch
from chattertools.models import Transaction

class TestClient(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        load_dotenv()
        cls.client = ch.Client(os.getenv('CHATTER_API_KEY'))

    def test_machine_list(self):
        machines = self.client.machines.list()
        self.assertTrue(len(machines) > 0)

    def test_machine_get(self):
        machines = self.client.machines.list()
        self.assertTrue(len(machines) > 0)
        machine = self.client.machines.get(machines[0].id)
        self.assertEqual(machine.id, machines[0].id)

    def test_machine_transaction_list(self):
        machines = self.client.machines.list()
        self.assertTrue(len(machines) > 0)
        transactions = machines[0].transactions.list()
        for t in transactions: print(t)
        self.assertTrue(len(transactions) > 0)

    def test_machine_transaction_create(self):
        machines = self.client.machines.list()
        self.assertTrue(len(machines) > 0)
        transaction = machines[0].transactions.create(None, ch.DataType.TOOL , 5)
        self.assertIsNotNone(transaction)
        self.assertEqual(transaction.dataType, ch.DataType.TOOL)
        self.assertEqual(transaction.value, 5)

    def test_machine_state_get(self):
        machines = self.client.machines.list()
        self.assertTrue(len(machines) > 0)
        state = machines[0].state.get()
        self.assertTrue(len(state) > 0)
        for s in state: print(s)


import requests

class ChatterRequestor:
    def __init__(self, key: str, baseUrl: str):
        self.key = key
        self.baseUrl = baseUrl

    def _getHeaders(self):
        return {
            "Authorization": f"Bearer {self.key}",
            "Content-Type": "application/json",
            "Accept": "application/json",
        }
    
    def get(self, path: str, params: dict = {}):
        response = requests.get(f"{self.baseUrl}{path}", headers=self._getHeaders(), params=params)
        self._checkResponse(response)
        return response.json()['data']
    
    def post(self, path: str, data: dict = {}):
        response = requests.post(f"{self.baseUrl}{path}", headers=self._getHeaders(), json=data)
        self._checkResponse(response)
        return response.json()['data']
    
    def put(self, path: str, data: dict = {}):
        response = requests.put(f"{self.baseUrl}{path}", headers=self._getHeaders(), json=data)
        self._checkResponse(response)
        return response.json()['data']
    
    def patch(self, path: str, data: dict = {}):
        response = requests.patch(f"{self.baseUrl}{path}", headers=self._getHeaders(), json=data)
        self._checkResponse(response)
        return response.json()['data']
    
    def delete(self, path: str):
        response = requests.delete(f"{self.baseUrl}{path}", headers=self._getHeaders())
        self._checkResponse(response)
        return response.json()['data']
    
    def _checkResponse(self, response):
        if response.status_code >= 400:
            raise ValueError(f"Request failed with status code {response.status_code}: {response.text}")




import chattertools as ch
import time

focas = ch.Focas(ip='10.0.2.51')

while True:
    print('Macro 101', focas.cnc_rdmacro(101))
    print('Macro 102', focas.cnc_rdmacro(102))
    print('Macro 103', focas.cnc_rdmacro(103))
    
    time.sleep(2)




from enum import Enum
from datetime import datetime
from ..helpers.parsers import parseDatetime, parseInt, parseBool

class DataType(Enum):
    MODE = (1, str)
    PROGRAM = (2, str)
    PART_COUNT = (3, int)
    PART_GOAL = (4, int)
    PALLET = (5, str)
    ALARM_ACTIVE = (6, bool)
    ALARM_CONTENTS = (7, str)
    MESSAGE_ACTIVE = (8, bool)
    MESSAGE_CONTENTS = (9, str)
    TOOL = (10, int)
    TAKT_TIME = (11, int)
    COMPLETION_TIME = (12, datetime)
    Q500 = (13, str)
    STATUS = (14, str)
    SPINDLE_SPEED = (21, int)
    COOLANT_LEVEL = (22, float)
    AIR_PRESSURE = (23, float)
    IN_CYCLE_TIME = (24, int)
    TOOL_TIMER = (25, int)
    INSPECTION_SIZE = (26, float)
    INSPECTION_TOLERANCE = (27, float)
    INSPECTION_SIZE_ERROR = (28, float)
    CUSTOM_DATA_1 = (29, str)
    CUSTOM_DATA_2 = (30, str)
    CUSTOM_DATA_3 = (31, str)
    IN_CYCLE = (32, bool)
    ONLINE = (33, bool)
    RAW_DEBUG = (34, str)
    PROGRAM_ID = (35, int)
    INSPECTION_X_POSITION = (36, float)
    INSPECTION_X_ERROR = (37, float)
    INSPECTION_Y_POSITION = (38, float)
    INSPECTION_Y_ERROR = (39, float)
    INSPECTION_Z_POSITION = (40, float)
    INSPECTION_Z_ERROR = (41, float)
    INSPECTION_FEATURE_ID = (42, float)
    INSPECTION_TRUE_POSITION_ERROR = (43, float)

    def __init__(self, value, type):
        self._value_ = value
        self.type = type

    @classmethod
    def fromValue(cls, value):
        for dt in cls:
            if dt._value_ == value:
                return dt
        raise ValueError(f"Unknown data type: {value}")
    
    @staticmethod
    def getParser(type):
        if type == int:
            return parseInt
        if type == bool:
            return parseBool
        if type == float:
            return float
        if type == datetime:
            return parseDatetime
        if type == str:
            return str
        return None


from typing import List
from ..models.machine import Machine
from .service import Service

class MachineService(Service):
    def __init__(self, requestor, *args, **kwargs):
        super().__init__(requestor, *args, **kwargs)

    def list(self, limit: int = 100, offset: int = 0) -> List[Machine]:
        """
        List all machines
        """
        params = {
            "limit": limit,
            "offset": offset
        }
        data = self._requestor.get("/machines", params=params)
        return [Machine.fromDict(self._requestor, machine) for machine in data]
    
    def get(self, id: int) -> Machine:
        """
        Get a machine by ID
        """
        data = self._requestor.get(f"/machines/{id}")
        return Machine.fromDict(self._requestor, data)


from typing import List
from ..models.transaction import Transaction
from .service import Service

from datetime import datetime

class StateService(Service):
    def __init__(self, requestor, machineId: int, *args, **kwargs):
        super().__init__(requestor, *args, **kwargs)
        
        self.machineId = machineId
    
    def get(self) -> List[Transaction]:
        """
        Get the state of a machine
        """
        data = self._requestor.get(f"/machines/{self.machineId}/state")
        return [Transaction.fromDict(d) for d in data]